# Django
from django import forms
from django.forms import ModelForm
from django.utils.safestring import mark_safe

# Local application / specific library imports
from .models import (
    InterpretationQuiz,
    InterpretationQuizQuestion,
    MoreInfoQuestion,
    MoreInfoQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizQuestion,
)


class PlainTextWidgetWithHiddenCopy(forms.Widget):
    def render(self, name, value, attrs=None, renderer=None):
        if hasattr(self, "initial"):
            value = self.initial

        return mark_safe(
            (str(value) if value is not None else "-")
            + f"<input type='hidden' name='{name}' value='{value}'>"
        )


class InterpretationQuizForm(ModelForm):
    class Meta:
        model = InterpretationQuiz
        fields = [
            "questions",
        ]

    questions = forms.ModelMultipleChoiceField(
        queryset=InterpretationQuizQuestion.objects.all(),
        widget=PlainTextWidgetWithHiddenCopy,
        required=False,
    )


class WeightedAnswersQuizForm(ModelForm):
    class Meta:
        model = WeightedAnswersQuiz
        fields = [
            "questions",
        ]

    questions = forms.ModelMultipleChoiceField(
        queryset=WeightedAnswersQuizQuestion.objects.all(),
        widget=PlainTextWidgetWithHiddenCopy,
        required=False,
    )


class MoreInfoQuizForm(ModelForm):
    class Meta:
        model = MoreInfoQuiz
        fields = [
            "questions",
        ]

    questions = forms.ModelMultipleChoiceField(
        queryset=MoreInfoQuestion.objects.all(),
        widget=PlainTextWidgetWithHiddenCopy,
        required=False,
    )



# Generated by Django 3.2.21 on 2023-10-03 15:29

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0002_auto_20230929_1725"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="moreinfoanswer",
            options={
                "verbose_name": "Réponse de quiz 'plus d'info'",
                "verbose_name_plural": "Réponses de quiz 'plus d'info'",
            },
        ),
        migrations.AlterModelOptions(
            name="moreinfoquestion",
            options={
                "verbose_name": "Question de quiz 'plus d'infos'",
                "verbose_name_plural": "Questions de quiz 'plus d'infos'",
            },
        ),
        migrations.AlterModelOptions(
            name="moreinfoquiz",
            options={
                "verbose_name": "Quiz plus d'infos",
                "verbose_name_plural": "Quizs plus d'infos",
            },
        ),
        migrations.AlterModelOptions(
            name="savedquiz",
            options={
                "verbose_name": "Quiz Sauvegardé",
                "verbose_name_plural": "Quiz Sauvegardés",
            },
        ),
        migrations.AddField(
            model_name="moreinfoquestion",
            name="answer_is_list",
            field=models.BooleanField(
                default=False,
                help_text="Check this parameter to display a big list of answers using a select input and not radio buttons.",
            ),
        ),
        migrations.AlterField(
            model_name="interpretationplugin",
            name="title_level",
            field=models.CharField(
                choices=[
                    ("h1", "Titre niveau 1 (<h1>)"),
                    ("h2", "Titre niveau 2 (<h2>)"),
                    ("h3", "Titre niveau 3 (<h3>)"),
                    ("h4", "Titre niveau 4 (<h4>)"),
                    ("h5", "Titre niveau 5 (<h5>)"),
                    ("h6", "Titre niveau 6 (<h6>)"),
                ],
                help_text="Title level",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="interpretationquiz",
            name="more_info_quiz",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Choisir un quiz 'plus d'infos' va l'afficher à la fin du quiz principal.",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="django_easy_quiz.moreinfoquiz",
            ),
        ),
        migrations.AlterField(
            model_name="moreinfoquestion",
            name="answer_is_txt",
            field=models.BooleanField(
                default=False,
                help_text="Cochez cette case pour autoriser les réponses 'texte libre' au lieu des réponses prédéfinies.",
            ),
        ),
        migrations.AlterField(
            model_name="moreinfoquiz",
            name="name",
            field=models.CharField(
                help_text="Sera affiché sous les résultats.", max_length=255
            ),
        ),
        migrations.AlterField(
            model_name="weightedanswersplugin",
            name="title_level",
            field=models.CharField(
                choices=[
                    ("h1", "Titre niveau 1 (<h1>)"),
                    ("h2", "Titre niveau 2 (<h2>)"),
                    ("h3", "Titre niveau 3 (<h3>)"),
                    ("h4", "Titre niveau 4 (<h4>)"),
                    ("h5", "Titre niveau 5 (<h5>)"),
                    ("h6", "Titre niveau 6 (<h6>)"),
                ],
                help_text="Title level",
                max_length=4,
            ),
        ),
        migrations.AlterField(
            model_name="weightedanswersquiz",
            name="more_info_quiz",
            field=models.ForeignKey(
                blank=True,
                default=None,
                help_text="Choisir un quiz 'plus d'infos' va l'afficher à la fin du quiz principal.",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="django_easy_quiz.moreinfoquiz",
            ),
        ),
    ]



# Generated by Django 3.2.21 on 2023-09-29 15:25

# Standard Library
import uuid

# Django
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="MoreInfoQuiz",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Will be displayed below the resuls.", max_length=255
                    ),
                ),
            ],
            options={
                "verbose_name": "More-Info Quiz",
                "verbose_name_plural": "More-Info Quizzes",
            },
        ),
        migrations.CreateModel(
            name="SavedQuiz",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("uuid", models.UUIDField(default=uuid.uuid4, editable=False)),
                ("quiz_type", models.CharField(max_length=255)),
                ("started_quiz", models.DateTimeField(auto_now_add=True)),
                ("quiz_ended", models.BooleanField(default=False)),
                ("answers", models.JSONField(default={})),
                ("conclusion", models.JSONField(default={})),
                ("more_infos", models.JSONField(default={})),
                (
                    "interpretation_quiz",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_easy_quiz.interpretationquiz",
                    ),
                ),
                (
                    "weighted_answers_quiz",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_easy_quiz.weightedanswersquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Saved Quiz",
                "verbose_name_plural": "Saved Quizzes",
            },
        ),
        migrations.CreateModel(
            name="MoreInfoQuestion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=255)),
                (
                    "answer_is_txt",
                    models.BooleanField(
                        default=False,
                        help_text="Check this parameter to allow plain text answers instead of pre-defined answers.",
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_easy_quiz.moreinfoquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "More-Info Question",
                "verbose_name_plural": "More-Info Questions",
            },
        ),
        migrations.CreateModel(
            name="MoreInfoAnswer",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("answer", models.CharField(max_length=512)),
                (
                    "question",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_easy_quiz.moreinfoquestion",
                    ),
                ),
            ],
            options={
                "verbose_name": "More-Info Answer",
                "verbose_name_plural": "More-Info Answers",
            },
        ),
        migrations.AddField(
            model_name="interpretationquiz",
            name="more_info_quiz",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="django_easy_quiz.moreinfoquiz",
            ),
        ),
        migrations.AddField(
            model_name="weightedanswersquiz",
            name="more_info_quiz",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="django_easy_quiz.moreinfoquiz",
            ),
        ),
    ]



# Generated by Django 3.2.21 on 2023-10-09 15:07

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_easy_quiz", "0003_auto_20231003_1729"),
    ]

    operations = [
        migrations.AlterField(
            model_name="savedquiz",
            name="answers",
            field=models.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name="savedquiz",
            name="conclusion",
            field=models.JSONField(default=dict),
        ),
        migrations.AlterField(
            model_name="savedquiz",
            name="more_infos",
            field=models.JSONField(default=dict),
        ),
    ]



# Generated by Django 3.2.20 on 2023-07-13 12:36

# Django
import django.db.models.deletion
from django.db import migrations, models

# Third party
import ckeditor_uploader.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
    ]

    operations = [
        migrations.CreateModel(
            name="InterpretationQuiz",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=1024, verbose_name="Nom du quiz")),
                (
                    "questions_random",
                    models.BooleanField(
                        default=False, verbose_name="Questions aléatoires"
                    ),
                ),
                ("create_date", models.DateTimeField(auto_now_add=True)),
                ("description", ckeditor_uploader.fields.RichTextUploadingField()),
                (
                    "conclusion_max_1",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Conclusion pour maximum de ■ :"
                    ),
                ),
                (
                    "conclusion_max_2",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Conclusion pour maximum de ▲ :"
                    ),
                ),
                (
                    "conclusion_max_3",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Conclusion pour maximum de ◆ :"
                    ),
                ),
                (
                    "conclusion_max_4",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Conclusion pour maximum de ● :"
                    ),
                ),
            ],
            options={
                "verbose_name": "Quiz d'interprétation",
                "verbose_name_plural": "Quiz d'interprétation",
            },
        ),
        migrations.CreateModel(
            name="WeightedAnswersQuiz",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=1024, verbose_name="Nom du quiz")),
                (
                    "questions_random",
                    models.BooleanField(
                        default=False, verbose_name="Questions aléatoires"
                    ),
                ),
                ("create_date", models.DateTimeField(auto_now_add=True)),
                ("description", ckeditor_uploader.fields.RichTextUploadingField()),
            ],
            options={
                "verbose_name": "Quiz avec réponses pondérées",
                "verbose_name_plural": "Quiz avec réponses pondérées",
            },
        ),
        migrations.CreateModel(
            name="WeightedAnswersQuizQuestion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=1024, verbose_name="Question")),
                (
                    "multiple_answers",
                    models.BooleanField(
                        default=False, verbose_name="Plusieurs réponses autorisées"
                    ),
                ),
                (
                    "answer_1",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Réponse 1"
                    ),
                ),
                (
                    "points_answer_1",
                    models.IntegerField(verbose_name="Points pour la réponse 1"),
                ),
                (
                    "answer_2",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 2"
                    ),
                ),
                (
                    "points_answer_2",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="Points pour la réponse 2"
                    ),
                ),
                (
                    "answer_3",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 3"
                    ),
                ),
                (
                    "points_answer_3",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="Points pour la réponse 3"
                    ),
                ),
                (
                    "answer_4",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 4"
                    ),
                ),
                (
                    "points_answer_4",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="Points pour la réponse 4"
                    ),
                ),
                (
                    "answer_5",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 5"
                    ),
                ),
                (
                    "points_answer_5",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="Points pour la réponse 5"
                    ),
                ),
                (
                    "answer_6",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 6"
                    ),
                ),
                (
                    "points_answer_6",
                    models.IntegerField(
                        blank=True, null=True, verbose_name="Points pour la réponse 6"
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_easy_quiz.weightedanswersquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Question de quiz avec réponses pondérées",
                "verbose_name_plural": "Questions de quiz avec réponses pondérées",
            },
        ),
        migrations.CreateModel(
            name="WeightedAnswersQuizConclusion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("description", ckeditor_uploader.fields.RichTextUploadingField()),
                (
                    "min_points",
                    models.IntegerField(
                        verbose_name="Montant minimal de points (inclus)"
                    ),
                ),
                (
                    "max_points",
                    models.IntegerField(
                        verbose_name="Montant maximal de points (inclus)"
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_easy_quiz.weightedanswersquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Conclusion de quiz avec réponses pondérées",
                "verbose_name_plural": "Conclusions de quiz avec réponses pondérées",
            },
        ),
        migrations.CreateModel(
            name="WeightedAnswersPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="django_easy_quiz_weightedanswersplugin",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "title_level",
                    models.CharField(
                        choices=[
                            ("h1", "Titre niveau 1 (<h1>)"),
                            ("h2", "Titre niveau 2 (<h2>)"),
                            ("h3", "Titre niveau 3 (<h3>)"),
                            ("h4", "Titre niveau 4 (<h4>)"),
                            ("h5", "Titre niveau 5 (<h5>)"),
                            ("h6", "Titre niveau 6 (<h6>)"),
                        ],
                        max_length=4,
                    ),
                ),
                (
                    "short_description",
                    models.CharField(
                        blank=True,
                        max_length=1024,
                        null=True,
                        verbose_name="Description courte",
                    ),
                ),
                (
                    "button_text",
                    models.CharField(
                        blank=True,
                        max_length=1024,
                        null=True,
                        verbose_name="Texte du bouton",
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="django_easy_quiz.weightedanswersquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Quiz avec réponses pondérées",
                "verbose_name_plural": "Quiz avec réponses pondérées",
            },
            bases=("cms.cmsplugin", models.Model),
        ),
        migrations.CreateModel(
            name="InterpretationQuizQuestion",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("label", models.CharField(max_length=1024, verbose_name="Question")),
                (
                    "answer_1",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        verbose_name="Réponse 1 (■)"
                    ),
                ),
                (
                    "answer_2",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 2 (▲)"
                    ),
                ),
                (
                    "answer_3",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 3 (◆)"
                    ),
                ),
                (
                    "answer_4",
                    ckeditor_uploader.fields.RichTextUploadingField(
                        blank=True, null=True, verbose_name="Réponse 4 (●)"
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_easy_quiz.interpretationquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Question de quiz d'interprétation",
                "verbose_name_plural": "Questions de quiz d'interprétation",
            },
        ),
        migrations.CreateModel(
            name="InterpretationPlugin",
            fields=[
                (
                    "cmsplugin_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        related_name="django_easy_quiz_interpretationplugin",
                        serialize=False,
                        to="cms.cmsplugin",
                    ),
                ),
                (
                    "title_level",
                    models.CharField(
                        choices=[
                            ("h1", "Titre niveau 1 (<h1>)"),
                            ("h2", "Titre niveau 2 (<h2>)"),
                            ("h3", "Titre niveau 3 (<h3>)"),
                            ("h4", "Titre niveau 4 (<h4>)"),
                            ("h5", "Titre niveau 5 (<h5>)"),
                            ("h6", "Titre niveau 6 (<h6>)"),
                        ],
                        max_length=4,
                    ),
                ),
                (
                    "short_description",
                    models.CharField(
                        blank=True,
                        max_length=1024,
                        null=True,
                        verbose_name="Description courte",
                    ),
                ),
                (
                    "button_text",
                    models.CharField(
                        blank=True,
                        max_length=1024,
                        null=True,
                        verbose_name="Texte du bouton",
                    ),
                ),
                (
                    "quiz",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="django_easy_quiz.interpretationquiz",
                    ),
                ),
            ],
            options={
                "verbose_name": "Quiz d'interprétation",
                "verbose_name_plural": "Quiz d'interprétation",
            },
            bases=("cms.cmsplugin", models.Model),
        ),
    ]



# Django
from django.utils.translation import gettext as _

# Third party
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

# Local application / specific library imports
from .models import InterpretationPlugin, WeightedAnswersPlugin


@plugin_pool.register_plugin
class WeightedAnswersPluginPublisher(CMSPluginBase):
    module = _("Quiz")
    name = _("Weighted Answers Quiz")
    model = WeightedAnswersPlugin
    render_template = "django_easy_quiz/weighted_answers_plugin.html"

    allow_children = False

    def render(self, context, instance, placeholder):
        context.update({"instance": instance})
        return context


@plugin_pool.register_plugin
class InterpretationPluginPublisher(CMSPluginBase):
    module = _("Quiz")
    name = _("Interpretation Quiz")
    model = InterpretationPlugin
    render_template = "django_easy_quiz/interpretation_plugin.html"
    allow_children = False

    def render(self, context, instance, placeholder):
        context.update({"instance": instance})
        return context



# Django
from django.contrib import admin, messages
from django.http import HttpResponseRedirect
from django.shortcuts import reverse
from django.utils.html import strip_tags
from django.utils.safestring import mark_safe
from django.utils.translation import gettext as _

# Third party
from import_export.admin import ExportActionModelAdmin

# Project
from django_easy_quiz.models import (
    InterpretationQuiz,
    InterpretationQuizQuestion,
    MoreInfoAnswer,
    MoreInfoQuestion,
    MoreInfoQuiz,
    SavedQuiz,
    WeightedAnswersQuiz,
    WeightedAnswersQuizConclusion,
    WeightedAnswersQuizQuestion,
)
from django_easy_quiz.settings import GATHER_STATISTICS
from django_easy_quiz.utils import (
    admin_error_lacks_answer,
    admin_error_min_max_conclusion,
)


def nb_of_questions(obj):
    try:
        return str(len(obj.weightedanswersquizquestion_set.all()))
    except AttributeError:
        return str(len(obj.interpretationquizquestion_set.all()))


class WeightedAnswersQuizConclusionAdminInline(admin.StackedInline):
    model = WeightedAnswersQuizConclusion
    extra = 1
    min_num = 1

    fieldsets = (
        (
            None,
            {
                "fields": (
                    ("min_points", "max_points"),
                    "description",
                ),
            },
        ),
    )


class WeightedAnswersQuizQuestionAdminInline(admin.StackedInline):
    model = WeightedAnswersQuizQuestion
    extra = 1
    min_num = 1

    fieldsets = (
        (
            None,
            {
                "fields": ("label", "multiple_answers"),
            },
        ),
        (
            _("Answer 1"),
            {
                "fields": (
                    "answer_1",
                    "points_answer_1",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Answer 2"),
            {
                "fields": (
                    "answer_2",
                    "points_answer_2",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Answer 3"),
            {
                "fields": (
                    "answer_3",
                    "points_answer_3",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Answer 4"),
            {
                "fields": (
                    "answer_4",
                    "points_answer_4",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Answer 5"),
            {
                "fields": (
                    "answer_5",
                    "points_answer_5",
                ),
                "classes": ("collapse",),
            },
        ),
        (
            _("Answer 6"),
            {
                "fields": (
                    "answer_6",
                    "points_answer_6",
                ),
                "classes": ("collapse",),
            },
        ),
    )


@admin.register(WeightedAnswersQuiz)
class WeightedAnswersQuizAdmin(admin.ModelAdmin):

    if GATHER_STATISTICS:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        ("name", "questions_random"),
                        "description",
                        "more_info_quiz",
                    ),
                },
            ),
        )
    else:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        ("name", "questions_random"),
                        "description",
                    ),
                },
            ),
        )

    list_display = (
        "id",
        "name",
        nb_of_questions,
        "questions_random",
    )
    list_display_links = (
        "id",
        "name",
    )
    list_filter = (
        "questions_random",
        "create_date",
    )
    search_fields = ("name",)

    inlines = [
        WeightedAnswersQuizQuestionAdminInline,
        WeightedAnswersQuizConclusionAdminInline,
    ]

    def save_formset(self, request, form, formset, change):
        """
        Save formset, BUT we will launch some checks before:
            - check that no conclusion have min_points > max_points
            - check that all conclusions create only one (enclosed) number sequence and that the min(answers_points_sum) & max(answers_points_sum) are within its bounds
        """
        instances = formset.save(commit=False)
        # don't know what those lines are doing
        for instance in instances:
            instance.save()

        # question formset is treated before conclusion, so in conclusion formset we can get the quiz, the questions, and the conclusion (using the formset.cleaned_data)
        if "WeightedAnswersQuizConclusionFormFormSet" in str(type(formset)):
            min_points_sum = 0
            max_points_sum = 0
            conclusions_combinations = []

            try:
                for question in (
                    formset[0]
                    .cleaned_data["quiz"]
                    .weightedanswersquizquestion_set.all()
                ):
                    all_points = [
                        question.points_answer_1,
                        question.points_answer_2,
                        question.points_answer_3
                        if question.points_answer_3 is not None
                        else 0,
                        question.points_answer_4
                        if question.points_answer_4 is not None
                        else 0,
                        question.points_answer_5
                        if question.points_answer_5 is not None
                        else 0,
                        question.points_answer_6
                        if question.points_answer_6 is not None
                        else 0,
                    ]

                    min_points_sum += min(all_points)
                    max_points_sum += max(all_points)
            except TypeError:
                nb_answers = 0
                if (
                    question.answer_1 is not (None or "")
                    and question.points_answer_1 is not None
                ):
                    nb_answers += 1
                if (
                    question.answer_2 is not (None or "")
                    and question.points_answer_2 is not None
                ):
                    nb_answers += 1
                if (
                    question.answer_3 is not (None or "")
                    and question.points_answer_3 is not None
                ):
                    nb_answers += 1
                if (
                    question.answer_4 is not (None or "")
                    and question.points_answer_4 is not None
                ):
                    nb_answers += 1
                if (
                    question.answer_5 is not (None or "")
                    and question.points_answer_5 is not None
                ):
                    nb_answers += 1
                if (
                    question.answer_6 is not (None or "")
                    and question.points_answer_6 is not None
                ):
                    nb_answers += 1
                admin_error_lacks_answer(request, question.label, nb_answers)

            for conclusion in formset.cleaned_data:
                if "min_points" in conclusion and "max_points" in conclusion:
                    if conclusion["min_points"] > conclusion["max_points"]:
                        # there is a problem, so we create a message with an extra tag using this function:
                        admin_error_min_max_conclusion(
                            request,
                            conclusion["min_points"],
                            conclusion["max_points"],
                            conclusion["description"],
                        )
                    conclusions_combinations.append(
                        [conclusion["min_points"], conclusion["max_points"]]
                    )
        # we need to save formset even if it have errors, because django needs its id somewhere later, and we don't want to make the user have to re-type the answers & conclusions
        formset.save_m2m()

    def response_add(self, request, obj, post_url_continue=None):
        """
        Method to get url to redirect after having created a new quiz:
            - Redirect to list view (using super) if there's no message with extra tag "easy_quiz_error".
            - Redirect to change view if a message with extra tag "easy_quiz_error" exist.
        """
        storage = messages.get_messages(request)
        for message in storage:
            if message.extra_tags == "easy_quiz_error":
                storage.used = False
                return HttpResponseRedirect(
                    reverse(
                        "admin:{}_{}_change".format(
                            obj._meta.app_label, obj._meta.model_name
                        ),
                        args=[obj.id],
                    )
                )
        storage.used = False
        return super().response_change(request, obj)

    def response_change(self, request, obj):
        """
        Method to get url to redirect after having modified a quiz:
            - Redirect to list view (using super) if there's no message with extra tag "easy_quiz_error".
            - Redirect to change view if a message with extra tag "easy_quiz_error" exist.
        """
        storage = messages.get_messages(request)
        for message in storage:
            if message.extra_tags == "easy_quiz_error":
                storage.used = False
                return HttpResponseRedirect(
                    reverse(
                        "admin:{}_{}_change".format(
                            obj._meta.app_label, obj._meta.model_name
                        ),
                        args=[obj.id],
                    )
                )
        storage.used = False

        return super().response_change(request, obj)


# InterpretationQuiz


class InterpretationQuizQuestionAdminInline(admin.StackedInline):
    model = InterpretationQuizQuestion
    extra = 1
    min_num = 1

    fieldsets = (
        (
            None,
            {
                "fields": (("label"),),
                "description": _("Do not forget to write conclusions!"),
            },
        ),
        (
            _("Answer 1 (■)"),
            {
                "classes": ("collapse",),
                "fields": ("answer_1",),
            },
        ),
        (
            _("Answer 2 (▲)"),
            {
                "classes": ("collapse",),
                "fields": ("answer_2",),
            },
        ),
        (
            _("Answer 3 (◆)"),
            {
                "classes": ("collapse",),
                "fields": ("answer_3",),
            },
        ),
        (
            _("Answer 4 (●)"),
            {
                "classes": ("collapse",),
                "fields": ("answer_4",),
            },
        ),
    )


@admin.register(InterpretationQuiz)
class InterpretationQuizAdmin(admin.ModelAdmin):
    if GATHER_STATISTICS:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        ("name", "questions_random"),
                        "description",
                        "more_info_quiz",
                    ),
                },
            ),
            (
                _("Conclusions"),
                {
                    "classes": ("collapse",),
                    "fields": (
                        "conclusion_max_1",
                        "conclusion_max_2",
                        "conclusion_max_3",
                        "conclusion_max_4",
                    ),
                },
            ),
        )
    else:
        fieldsets = (
            (
                None,
                {
                    "fields": (
                        ("name", "questions_random"),
                        "description",
                    ),
                },
            ),
            (
                _("Conclusions"),
                {
                    "classes": ("collapse",),
                    "fields": (
                        "conclusion_max_1",
                        "conclusion_max_2",
                        "conclusion_max_3",
                        "conclusion_max_4",
                    ),
                },
            ),
        )

    list_display = (
        "id",
        "name",
        nb_of_questions,
        "questions_random",
    )
    list_display_links = (
        "id",
        "name",
    )
    list_filter = (
        "questions_random",
        "create_date",
    )
    search_fields = ("name",)

    inlines = [
        InterpretationQuizQuestionAdminInline,
    ]


# activate stats only if activated in settings (default = False)
if GATHER_STATISTICS:

    @admin.register(SavedQuiz)
    class SavedQuizAdmin(ExportActionModelAdmin, admin.ModelAdmin):
        def has_change_permission(self, request, obj=None):
            return False

        list_display = ("quiz_type", "quiz_ended", "uuid")

        fields = (
            "quiz_type",
            "quiz_ended",
            "answers_admin",
            "conclusion_points_admin",
            "conclusion_admin",
            "more_infos_admin",
        )

        @admin.display(description=_("Answers"))
        def answers_admin(self, obj):
            answers_txt = "<ul>"
            for answers in obj.answers:
                answers_txt += f"<li>{answers['question']['label']}<ul>"
                if "answers" in answers:  # weighted answers quiz
                    for answer in answers["answers"]:
                        answers_txt += f"<li>{strip_tags(answer)}</li>"
                else:
                    answers_txt += f"<li>{strip_tags(answers['question']['answer_1'])}</li><li>{strip_tags(answers['question']['answer_2'])}</li><li>{strip_tags(answers['question']['answer_3'])}</li><li>{strip_tags(answers['question']['answer_4'])}</li>"
                answers_txt += "</ul></li>"
            if answers_txt == "<ul>":
                answers_txt += f"<li>{_('No answer recorded for this quiz.')}</li>"
            answers_txt += "</ul>"
            return mark_safe(answers_txt)

        @admin.display(description=_("Points"))
        def conclusion_points_admin(self, obj):
            if "points" in obj.conclusion:  # weighted_answers quiz
                return obj.conclusion["points"]
            return ""  # interpretation quiz

        @admin.display(description=_("Conclusion"))
        def conclusion_admin(self, obj):
            if "description" in obj.conclusion:  # weighted_answers quiz
                return mark_safe(obj.conclusion["description"])
            return mark_safe(obj.conclusion)  # interpretation quiz

        @admin.display(description=_("More infos"))
        def more_infos_admin(self, obj):
            more_infos_txt = "<ul>"
            for question in obj.more_infos:
                more_infos_txt += f"<li>{question['question']}<ul>"
                more_infos_txt += f"<li>{question['answer']}</li>"
                more_infos_txt += "</ul></li>"
            return mark_safe(more_infos_txt)


# activate more info questions only if activated in settings (default = False)
if GATHER_STATISTICS:

    class MoreInfoAnswerAdminInline(admin.StackedInline):
        model = MoreInfoAnswer

    @admin.register(MoreInfoQuestion)
    class MoreInfoQuestionAdmin(admin.ModelAdmin):
        inlines = [
            MoreInfoAnswerAdminInline,
        ]

        readonly_fields = ("quiz",)

    class MoreInfoQuestionAdminInline(admin.StackedInline):
        model = MoreInfoQuestion
        show_change_link = True

    @admin.register(MoreInfoQuiz)
    class MoreInfoQuizAdmin(admin.ModelAdmin):

        fieldsets = (
            (
                None,
                {
                    "fields": (("name"),),
                },
            ),
            (
                None,
                {
                    "fields": [],
                    "description": _(
                        "Save your questions first, and then you will be able to update each question to add answers."
                    ),
                },
            ),
        )

        inlines = [
            MoreInfoQuestionAdminInline,
        ]



# Standard Library
from uuid import uuid4

# Django
from django.db import models
from django.utils.translation import gettext as _

# Third party
from ckeditor_uploader.fields import RichTextUploadingField
from cms.models.pluginmodel import CMSPlugin

# Quiz


class Quiz(models.Model):
    name = models.CharField(_("Quiz name"), max_length=1024)
    questions_random = models.BooleanField(
        default=False, verbose_name=_("Random questions")
    )
    create_date = models.DateTimeField(auto_now_add=True)
    description = RichTextUploadingField()
    more_info_quiz = models.ForeignKey(
        "MoreInfoQuiz",
        default=None,
        blank=True,
        null=True,
        on_delete=models.PROTECT,
        help_text=_("Will display this 'more info quiz' at the end of the main quiz."),
    )

    def __str__(self):
        return f"{self.name}"

    class Meta:
        abstract = True


class WeightedAnswersQuiz(Quiz):
    class Meta:
        verbose_name = _("Weighted Answers Quiz")
        verbose_name_plural = _("Weighted Answers Quizzes")


class InterpretationQuiz(Quiz):

    conclusion_max_1 = RichTextUploadingField(
        verbose_name=_("Conclusion for max of ■:")
    )
    conclusion_max_2 = RichTextUploadingField(
        verbose_name=_("Conclusion for max of ▲:")
    )
    conclusion_max_3 = RichTextUploadingField(
        verbose_name=_("Conclusion for max of ◆:")
    )
    conclusion_max_4 = RichTextUploadingField(
        verbose_name=_("Conclusion for max of ●:")
    )

    class Meta:
        verbose_name = _("Interpretation Quiz")
        verbose_name_plural = _("Interpretation Quizzes")


# Questions


class Question(models.Model):
    label = models.CharField(_("Question"), max_length=1024)

    class Meta:
        abstract = True


class WeightedAnswersQuizQuestion(Question):
    quiz = models.ForeignKey(WeightedAnswersQuiz, on_delete=models.CASCADE)
    label = models.CharField(verbose_name=_("Question"), max_length=1024)
    multiple_answers = models.BooleanField(
        default=False, verbose_name=_("More than one answer possible")
    )

    answer_1 = RichTextUploadingField(verbose_name=_("Answer 1"))
    points_answer_1 = models.IntegerField(verbose_name=_("Points for answer 1"))
    answer_2 = RichTextUploadingField(verbose_name=_("Answer 2"), null=True, blank=True)
    points_answer_2 = models.IntegerField(
        verbose_name=_("Points for answer 2"), null=True, blank=True
    )
    answer_3 = RichTextUploadingField(verbose_name=_("Answer 3"), null=True, blank=True)
    points_answer_3 = models.IntegerField(
        verbose_name=_("Points for answer 3"), null=True, blank=True
    )
    answer_4 = RichTextUploadingField(verbose_name=_("Answer 4"), null=True, blank=True)
    points_answer_4 = models.IntegerField(
        verbose_name=_("Points for answer 4"), null=True, blank=True
    )
    answer_5 = RichTextUploadingField(verbose_name=_("Answer 5"), null=True, blank=True)
    points_answer_5 = models.IntegerField(
        verbose_name=_("Points for answer 5"), null=True, blank=True
    )
    answer_6 = RichTextUploadingField(verbose_name=_("Answer 6"), null=True, blank=True)
    points_answer_6 = models.IntegerField(
        verbose_name=_("Points for answer 6"), null=True, blank=True
    )

    def __str__(self):
        quiz_name = ""
        if self.quiz is not None:
            quiz_name = _("[in quizz ") + self.quiz.name + "] "
        return f"{quiz_name}{self.label}"

    def toJson(self):
        return {
            "label": self.label,
            "multiple_answers": self.multiple_answers,
            "answer_1": self.answer_1,
            "answer_2": self.answer_2,
            "answer_3": self.answer_3,
            "answer_4": self.answer_4,
            "answer_5": self.answer_5,
            "answer_6": self.answer_6,
            "points_answer_1": self.points_answer_1,
            "points_answer_2": self.points_answer_2,
            "points_answer_3": self.points_answer_3,
            "points_answer_4": self.points_answer_4,
            "points_answer_5": self.points_answer_5,
            "points_answer_6": self.points_answer_6,
        }

    class Meta:
        verbose_name = _("Weighted Answers Quiz Question")
        verbose_name_plural = _("Weighted Answers Quiz Questions")


class InterpretationQuizQuestion(Question):
    quiz = models.ForeignKey(InterpretationQuiz, on_delete=models.CASCADE)
    label = models.CharField(verbose_name=_("Question"), max_length=1024)

    answer_1 = RichTextUploadingField(verbose_name=_("Answer 1 (■)"))
    answer_2 = RichTextUploadingField(
        verbose_name=_("Answer 2 (▲)"), null=True, blank=True
    )
    answer_3 = RichTextUploadingField(
        verbose_name=_("Answer 3 (◆)"), null=True, blank=True
    )
    answer_4 = RichTextUploadingField(
        verbose_name=_("Answer 4 (●)"), null=True, blank=True
    )

    def __str__(self):
        quiz_name = ""
        if self.quiz is not None:
            quiz_name = _("[in quizz ") + self.quiz.name + "] "
        return f"{quiz_name}{self.label}"

    def toJson(self):
        return {
            "label": self.label,
            "answer_1": f"■ - {self.answer_1}",
            "answer_2": f"▲ - {self.answer_2}",
            "answer_3": f"◆ - {self.answer_3}",
            "answer_4": f"● - {self.answer_4}",
        }

    class Meta:
        verbose_name = _("Interpretation Quiz Question")
        verbose_name_plural = _("Interpretation Quiz Questions")


# Answers


class Answer(models.Model):
    answer = RichTextUploadingField()

    def __str__(self):
        return _(f"Answer #{self.id}")

    class Meta:
        abstract = True


INTERPRETATION_QUESTION_CHOICES = (
    ("1", "■"),
    ("2", "▲"),
    ("3", "◆"),
    ("4", "●"),
)

# Conclusions/results


class QuizConclusion(models.Model):
    description = RichTextUploadingField()

    class Meta:
        abstract = True


class WeightedAnswersQuizConclusion(QuizConclusion):
    quiz = models.ForeignKey(WeightedAnswersQuiz, on_delete=models.CASCADE)
    min_points = models.IntegerField(_("Min amount of points (included)"))
    max_points = models.IntegerField(_("Max amount of points (included)"))

    def __str__(self):
        return _("Weighted Answers Quiz Conclusion") + " #" + str(self.id)

    class Meta:
        verbose_name = _("Weighted Answers Quiz Conclusion")
        verbose_name_plural = _("Weighted Answers Quiz Conclusions")


# Djangocms Plugins


class BasePlugin(models.Model):
    LEVEL_1 = "h1"
    LEVEL_2 = "h2"
    LEVEL_3 = "h3"
    LEVEL_4 = "h4"
    LEVEL_5 = "h5"
    LEVEL_6 = "h6"
    TITLE_LEVEL = [
        (LEVEL_1, _("Title level 1 (<h1>)")),
        (LEVEL_2, _("Title level 2 (<h2>)")),
        (LEVEL_3, _("Title level 3 (<h3>)")),
        (LEVEL_4, _("Title level 4 (<h4>)")),
        (LEVEL_5, _("Title level 5 (<h5>)")),
        (LEVEL_6, _("Title level 6 (<h6>)")),
    ]

    title_level = models.CharField(
        choices=TITLE_LEVEL, max_length=4, help_text=_("Title level")
    )
    short_description = models.CharField(
        _("Short description"),
        max_length=1024,
        null=True,
        blank=True,
    )
    button_text = models.CharField(
        _("Button text"),
        max_length=1024,
        null=True,
        blank=True,
    )

    class Meta:
        abstract = True


class WeightedAnswersPlugin(BasePlugin, CMSPlugin):
    quiz = models.ForeignKey(WeightedAnswersQuiz, on_delete=models.SET_NULL, null=True)

    class Meta:
        verbose_name = _("Weighted Answers Quiz")
        verbose_name_plural = _("Weighted Answers Quizzes")


class InterpretationPlugin(BasePlugin, CMSPlugin):
    quiz = models.ForeignKey(InterpretationQuiz, on_delete=models.SET_NULL, null=True)

    class Meta:
        verbose_name = _("Interpretation Quiz")
        verbose_name_plural = _("Interpretation Quizzes")


class SavedQuiz(models.Model):
    uuid = models.UUIDField(default=uuid4, editable=False)
    quiz_type = models.CharField(max_length=255)
    started_quiz = models.DateTimeField(auto_now_add=True)
    quiz_ended = models.BooleanField(default=False)
    answers = models.JSONField(default=dict)
    weighted_answers_quiz = models.ForeignKey(
        WeightedAnswersQuiz, null=True, blank=True, on_delete=models.PROTECT
    )
    interpretation_quiz = models.ForeignKey(
        InterpretationQuiz, null=True, blank=True, on_delete=models.PROTECT
    )
    conclusion = models.JSONField(default=dict)
    more_infos = models.JSONField(default=dict)

    class Meta:
        verbose_name = _("Saved Quiz")
        verbose_name_plural = _("Saved Quizzes")


class MoreInfoQuiz(models.Model):
    name = models.CharField(
        max_length=255, help_text=_("Will be displayed below the resuls.")
    )

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _("More-Info Quiz")
        verbose_name_plural = _("More-Info Quizzes")


class MoreInfoQuestion(models.Model):
    label = models.CharField(max_length=255)
    quiz = models.ForeignKey(MoreInfoQuiz, on_delete=models.PROTECT)
    answer_is_txt = models.BooleanField(
        default=False,
        help_text=_(
            "Check this parameter to allow plain text answers instead of pre-defined answers."
        ),
    )
    answer_is_list = models.BooleanField(
        default=False,
        help_text=_(
            "Check this parameter to display a big list of answers using a select input and not radio buttons."
        ),
    )

    def __str__(self):
        return self.label

    class Meta:
        verbose_name = _("More-Info Question")
        verbose_name_plural = _("More-Info Questions")


class MoreInfoAnswer(models.Model):
    question = models.ForeignKey(MoreInfoQuestion, on_delete=models.PROTECT)
    answer = models.CharField(max_length=512)

    def __str__(self):
        return self.answer

    class Meta:
        verbose_name = _("More-Info Answer")
        verbose_name_plural = _("More-Info Answers")



# Standard Library
from random import shuffle

# Django
from django.contrib import messages
from django.forms import formset_factory
from django.utils.translation import gettext as _

# Local application / specific library imports
from .forms import MoreInfoQuizForm


def admin_error_min_max_conclusion(request, min, max, clue):
    text = _(f"Error: {min} (min value) > {max} (max value). Description: {clue}")
    messages.set_level(request, messages.ERROR)
    messages.error(request, text, extra_tags="easy_quiz_error")


def admin_error_lacks_answer(request, question, nb_answers):
    text = _(
        f'Error: question "{question}" lacks {4-nb_answers} answer(s) (answer text or nb of points).'
    )
    messages.set_level(request, messages.ERROR)
    messages.error(request, text, extra_tags="easy_quiz_error")


def get_quiz_and_formset(QuizType, FormType, pk):
    quiz = QuizType.objects.get(id=int(pk))
    initial_data = []

    try:
        questions = quiz.interpretationquizquestion_set.all()
        for question in questions:
            answers = []
            if question.answer_1 is not (None or ""):
                answers.append({"label": question.answer_1, "id": 1, "symbol": "■"})
            if question.answer_2 is not (None or ""):
                answers.append({"label": question.answer_2, "id": 2, "symbol": "▲"})
            if question.answer_3 is not (None or ""):
                answers.append({"label": question.answer_3, "id": 3, "symbol": "◆"})
            if question.answer_4 is not (None or ""):
                answers.append({"label": question.answer_4, "id": 4, "symbol": "●"})
            shuffle(answers)
            initial_data.append({"question": question, "answers": answers})

    except AttributeError:
        questions = quiz.weightedanswersquizquestion_set.all()
        for question in questions:
            answers = []
            if question.answer_1 is not (None or ""):
                answers.append({"label": question.answer_1, "id": 1})
            if question.answer_2 is not (None or ""):
                answers.append({"label": question.answer_2, "id": 2})
            if question.answer_3 is not (None or ""):
                answers.append({"label": question.answer_3, "id": 3})
            if question.answer_4 is not (None or ""):
                answers.append({"label": question.answer_4, "id": 4})
            if question.answer_5 is not (None or ""):
                answers.append({"label": question.answer_5, "id": 5})
            if question.answer_6 is not (None or ""):
                answers.append({"label": question.answer_6, "id": 6})
            shuffle(answers)
            initial_data.append({"question": question, "answers": answers})

    if quiz.questions_random:
        questions = questions.order_by("?")
    QuestionsFormSet = formset_factory(FormType, extra=0, can_delete=False)

    formset = QuestionsFormSet(initial=initial_data, prefix="questions")

    return quiz, formset


def get_more_info_quiz_formset(more_info_quiz):
    initial_data = []
    questions = more_info_quiz.moreinfoquestion_set.all()

    for question in questions:
        answers = [
            {"label": answer.answer, "id": answer.id}
            for answer in question.moreinfoanswer_set.all()
        ]
        initial_data.append({"question": question, "answers": answers})

    MoreInfoQuestionsFormSet = formset_factory(
        MoreInfoQuizForm, extra=0, can_delete=False
    )
    formset = MoreInfoQuestionsFormSet(initial=initial_data, prefix="questions")

    return formset
